@@ -1,9 +1,10 @@
 package Unzip::Passwd;
 use Moose;
+use Data::Dumper;
 
 has filename 	=> (is => 'rw'); #nome do arquivo zip
 has passwd		=> (is => 'rw'); #senha
-has destiny		=> (is => 'rw'); #caminho para o destino do arquivo
+has destiny		=> (is => 'rw' ,default => './'); #caminho para o destino do arquivo
 has errors		=> (is => 'rw'); #atributo que guarda os erros.
 has debug		=> (is => 'rw' , default => 0); #atributo que guarda os erros.
 
@@ -19,11 +20,11 @@ has debug		=> (is => 'rw' , default => 0); #atributo que guarda os erros.
 
 =head1 VERSION
 
-Version 0.0.8
+Version 0.0.9
 
 =cut
 
-our $VERSION = '0.0.8';
+our $VERSION = '0.0.9';
 
 
 =head1 SYNOPSIS
@@ -39,93 +40,97 @@ our $VERSION = '0.0.8';
  #done!
 
 
-=head2 METHOD
+=head1 METHODS
 
 
 =head2 unzip
 
- Do the job, basicly. But first invokes the analyze method, to have certain the zip file is fine. 
- No parameters, just return 1 if it's all ok. Otherwise, will return 0 and trhow an exception.
-
-
+ Do the job, basicly. But first invokes the analyze method, to have certain the zip file is fine.
+ if analyze returns 1, then unzip will try to open the zip file.
+ No parameters, will return 1 if it's all ok. Otherwise, will return 0 and throw an exception.
 
 =cut
 
 sub unzip {
 	my ( $self ) 	= @_;
 	my @errors 		= ();
+	my $ok			= 0;
 	if( !$self->filename ){
-		push @errors, 'You must enter with a zip file name.';
-	}
-	elsif(! -e $self->filename ){
-		push @errors, 'O arquivo "' . $self->filename . '" não existe!';
-	}
-	my @c = readpipe 'unzip -l ' . $self->filename;
-	my @files = ();
-	foreach(@c) {
-			if( $_ =~ /\d+.+?\d+-\d+-\d+ \d\d:\d\d(\ |\t)+(.+?)$/ ){
-				push @files, $2  
-			}
-	}
-	print "\nFILES: \n";
-
-	#preparando arquivos para descompactar. Evitando mensagens de confirmação...
-	if(@files > 0){
-		$self->analyze(\@files);
-	}
-	elsif(-e $self->filename and @files == 0){
-		push @errors, 'the zip file "' . $self->filename . '" is empty!';
+		push @errors, "You must define the 'filename' attribute.";
 	}
 	else {
-		push @errors, 'The file "' . $self->filename . '" not exists!';
+		my @files = @{$self->list_files};
+		if(@files){
+			#prepare files and directories(if exists) to extract. This avoids confirmation messages too.
+			if(@errors == 0 && @files > 0 && $self->analyze){
+				$self->execute;
+			}
+			else {
+				$self->show_errors;
+			}
+			print "\n";
+		}
+		else {
+			push @errors, 'the zip file "' . $self->filename . '" is empty!';
+		}
 	}
-	print "\n";
-	
 	if( @errors > 0 ){
-		foreach my $e( @errors ) {
-			print "\nERROR: $e";
+		if(ref( $self->errors ) =~ /ARRAY/){
+			my @_errors 	= @{$self->errors} || ();
+			push @errors , @_errors;
+			$self->errors(\@errors);
+			$self->show_errors;
+		}
+		else{
+			$self->errors(\@errors);
+			$self->show_errors;
 		}
-		$self->errors(\@errors);
-		return 0;
 	}
 	else{
-		return $self->exec_unzip;
+		$ok = 1; 
 	}
+	return $ok;
 }
 
 
-=head2 show_errors
+=head2 list_files
 
- Makes the obvious. Just show errors. Don't receives anything. Returns the error messages( arrayref ).
+ try to obtain a list of files from zipfile in $self->filename. If succeded, returns an arrayref with the filelist. Otherwise returns 
+ an arrayref empty. 
 
 =cut
 
-sub show_errors {
+sub list_files {
 	my ( $self ) = @_;
-	my $m = '';
-	my @errors = @{$self->errors};
-	if(ref($self->errors) =~ /ARRAY/){
-		foreach my $e(@{$self->errors}){
-			push @errors, $e;
+	my @files = ();
+	my @errors = ();
+	if(-e $self->filename){
+		my @c = readpipe 'unzip -l ' . $self->filename;
+		@files = ();
+		foreach(@c) {
+			#extracting just the filename. It's all the matters.
+			if( $_ =~ /\d+.+?\d+-\d+-\d+ \d\d:\d\d(\ |\t)+(.+?)$/ ){
+				push @files, $2 ;
+			}
 		}
 	}
-	else{
-		print "\nNo errors! :D\n";
-		@errors = ();
+	else {
+		push @errors , "The file defined in 'filename' attribute doesn't exists!";
+		my @objerrors = ();
+		if(defined( $self->errors ) and ref( $self->errors ) =~ /ARRAY/){
+			@objerrors = @{$self->errors};
+			push @objerrors , @errors;
+			$self->errors( @objerrors );
+		}
 	}
-	if(@errors){
-		my @objerrors = $self->errors;
-		push @objerrors , @errors;
-		$self->errors(\@errors);
-	}
-	return \@errors;
+	return \@files;
 }
 
 
 =head2 analyze
 
- Analyze possible file and directory problems( permissions non-existing directories etc ). return 1 if 
- all it's ok! Otherwise return 0. Receives the files list( arrayref ) as parameter.
+ Analyzes possible file and directory problems( permissions and non-existing directories etc ). Returns 1 if 
+ all it's ok! Otherwise returns 0. Receives the files list( arrayref ) as parameter.
 
 =cut
 
@@ -141,10 +146,11 @@ sub analyze {
 			$dirfile = $self->destiny . '/' . $file ;
 		}
 		else{
-			$self->destiny( undef );
+			$self->destiny( './' );
 		}
-		if(-e $file && !defined($self->destiny) ){
-			print "\nDELETING $file (already exists)";
+		$file = $self->destiny . '/' . $file;
+		if(-e $file ){
+			print "\nOVERWRITING $file (already exists)";
 			eval{$ok = unlink $file};
 			if($@){
 				push @errors, $@;
@@ -154,34 +160,28 @@ sub analyze {
 				$ok = 1;
 			}
 		}
-		elsif(!-e $file && !defined($self->destiny)){
-			print "\nERROR: The file or directory '$file' not exists!";
-			push @errors , "ERROR: O arquivo ou diretório $file não existe!" ;
-		}
-		elsif(defined($self->destiny) && length($self->destiny) > 0 ) {
-			print "\nDELETING $dirfile (already exists)";
-			eval{unlink $dirfile;};
-			if($@){
-				push @errors, $@;
-			}
-			else{
-				print "\nOK!";
-				$ok = 1;
-			}
+		else {
+			push @errors , "The file and/or destination can't be readed '" . $self->destiny . $file . "'";
 		}
 	}
 
-	#This is weard... I don't know how to get better... :p
+	#This is weard... I really need to do this better... :(
 	if(@errors > 0){
-		my @objerrors = @{$self->errors};
-		push @objerrors , @errors;
-		$self->errors( \@objerrors );
+		my @objerrors = ();
+		if(defined( $self->errors ) and ref( $self->errors ) =~ /ARRAY/){
+			@objerrors = @{$self->errors};
+			push @objerrors , @errors;
+			$self->errors( \@objerrors );
+		}
+		$ok = 0;
 	}
 	return $ok;
 }
 
 
-#This is who really do the job... Takes the options and agreggate to command before execute.
+# Internal method. 
+# This is who really do the job... Takes the options and agreggate to command before execute. 
+# Returns 1 if all is right. Otherwise returns 0.
 sub exec_unzip {
 	my ( $self ) = @_;
 	#the command obviously starts with unzip...
@@ -189,7 +189,9 @@ sub exec_unzip {
 	my @errors = ();
 	my @commands = ();
 
-	#if the password is defined and have some password... -P is added
+	#THIS PART AGREGATE OPTIONS FROM CONFIG( see the constructor method ), BEFORE EXECUTE.
+
+	#if the password is defined and have some password... -P is added to command
 	if(defined($self->passwd) and length($self->passwd) > 0){
 		$comm .= ' -P ' . $self->passwd . ' ';
 	}
@@ -205,8 +207,9 @@ sub exec_unzip {
 	else {
 		push @errors,'The size MUST be bigger than 0';
 	}
-
 	my $target = $self->filename;
+
+	#FINALY!!! EXECUTE THIS SHIT!!!
 	if($comm !~ /$target/){
 		$comm .= " $target";
 		@commands = readpipe $comm;
@@ -229,22 +232,59 @@ sub exec_unzip {
 }
 
 
+=head2 show_errors
 
-=head2 AUTHOR
+ Makes the obvious. Show errors. Don't receives anything. Returns the error messages( arrayref ).
+
+=cut
+
+sub show_errors {
+	my ( $self ) = @_;
+	my $m = '';
+	my @errors = (); 
+	if( ref($self->errors) =~ /ARRAY/ ){
+		@errors = @{$self->errors};
+		foreach my $e( @errors ){
+			print STDERR "\nERROR: $e";
+		}
+	}
+	else {
+		print STDERR "\nNo errors! :D\n";
+		@errors = ();
+	}
+	return \@errors;
+}
+
+
+=head2 DEPENDECIES
+
+
+ NO DEPENDECIES!!! But unzip program MUST BE INSTALLED in your O.S.
+
+
+=head1 AUTHOR
 
 Andre Carneiro, C<< <andregarciacarneiro at gmail.com> >>
 
+
+=head1 NOTES FOR THIS VERSION
+
+
+ Many changes! The code is more optimized and less complex. Some basic tests are implemented ( finaly! :P ).
+
+
 =head1 BUGS
 
+
 Please report any bugs or feature requests to C<bug-unzip-passwd at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Unzip-Passwd>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 
-
-
 =head1 SUPPORT
 
+This module was tested JUST ON LINUX. DON'T HAVE SUPPORT IN WINDOWS YET. MAYBE LATER...
+
 You can find documentation for this module with the perldoc command.
 
     perldoc Unzip::Passwd
@@ -278,7 +318,9 @@ L<http://search.cpan.org/dist/Unzip-Passwd/>
 
  Aggregates some log module.
 
- Create tests... :(
+ Finish the tests... :(
+
+ Create a better way to treat exceptions...
 
 =head1 ACKNOWLEDGEMENTS
 
